'''
Python module used in InfectionExplorer
'''
import random


HOSPITAL_CAPACITY = 0.15

class Person():
    social_distancing = 3
    sick_fraction = 0.0
    school_coords = (140, 260)
    store_coords = (375, 260)
    work_coords = (610, 260)

    def __init__(self, id: int, age: int=50, existing_conditions: bool=False):
        self.id = id
        self.walk_rate = 4 * (self.id % 6) + 20
        self.age = age
        self.existing_conditions = existing_conditions
        self.home_coords = (None, None)
        self.x, self.y = None, None
        self.target_location = (None, None)
        self.reset_target_location()
        self.condition: str='healthy'  # one of 'healthy', 'asymptomatic', 'sick', 'critical', 'recovered', 'dead'
        self.initial_condition = 'healthy'
        self.infection_day: int=0
        if self.id % 20 == 10: #random.random() < 0.02:
            self.infect(1)
            self.initial_condition = self.condition
        self._move = self.move  # save reference to original move function


    def infect(self, day):
        self.infection_day = day
        if random.random() < 0.5:
            self.condition = 'asymptomatic'
        else:
            self.condition = 'sick'

    def reset(self):
        self.x, self.y = self.home_coords
        self.target_location = self.home_coords
        self.y = (self.y + 2) * 2.5 + 20 * (random.random() - 0.5)
        self.condition = self.initial_condition
        self.move = self._move  # reset move func (so previously dead persons can move again)

    def update_condition(self, population: list, day: int, contagious_persons: dict):
        # print(self.condition)
        if self.condition == 'healthy':
            if self.y > 40:
                # check distance to all other Persons and see if this person becomes infected
                self_x, self_y = self.x, self.y
                for person in contagious_persons:
                    if abs(self_x - person.x) < 25 and abs(self_y - person.y) < 25:  # quick check before heavier distance calc
                        distance = ((self_x - person.x) ** 2 + (self_y - person.y) ** 2) ** 0.5
                        if random.random() < (0.3 - distance / 75):
                            self.infect(day)
                            break
            else:  # self.y <= 40:
                self_x = self.x
                for person in contagious_persons:
                    if person.x == self_x:
                        distance = abs(self.y - person.y)
                        if distance < 8 and random.random() < 0.18:
                            self.infect(day)
                            break
        elif self.condition == 'asymptomatic':
            if random.randint(3, 10) < day - self.infection_day:
                self.condition = 'sick'
            elif random.random() < 0.05 and day - self.infection_day > 2:
                self.condition = 'recovered'
        elif self.condition == 'sick':
            if random.random() < 0.05 and random.randint(0, 120) < self.age and day - self.infection_day > 3:
                self.condition = 'critical'
            elif self.sick_fraction > HOSPITAL_CAPACITY and random.random() < 0.15:
                self.condition = 'critical'
            elif random.random() < 0.05 and day - self.infection_day > 5:
                self.condition = 'recovered'
        elif self.condition == 'critical':
            if random.randint(0, 120) < self.age and random.random() < 0.5:
                self.condition = 'dead'
            elif self.sick_fraction > HOSPITAL_CAPACITY and random.random() < 0.05:
                self.condition = 'dead'
            elif random.random() < 0.1 and self.sick_fraction < HOSPITAL_CAPACITY:
                self.condition = 'sick'


    def move(self, tick: int, day: int):
        if self.condition == 'dead':
            self.x, self.y = 750 + 7 * (self.id % 7), 6 + 15 * self.id // 10
            self.move = lambda *args: None  # reset func to not evauate any additional calls
        else:
            if random.random() < 0.4:
                xmove = random.randint(-13, 13)
                ymove = random.randint(-13, 13)

            else:
                targ_x, targ_y = self.target_location
                xmove, ymove = 0, 0
                walk_rate = self.walk_rate

                if self.x < (targ_x - 40):
                    xmove += walk_rate  # random.randint(5, 12)
                elif self.x >= targ_x + 40:
                    xmove -= walk_rate  # random.randint(5, 12)

                if self.y < (targ_y - 40):
                    ymove += walk_rate  # random.randint(5, 12)
                elif self.y >= (targ_y + 40):
                    ymove -= walk_rate  # random.randint(5, 12)

            if self.y > 40:
                self.x += xmove
                self.y += ymove
            else:
                self.x = self.home_coords[0]
                self.y += ymove // 2
                if self.y < 7:
                    self.y = 8


    def reset_target_location(self, home: bool=False) -> tuple:
        if not home:
            if random.random() > self.social_distancing / 6 + 0.1:
                if random.random() < 0.1:
                    self.target_location = (random.randint(20, 650), random.randint(80, 350))  # random spot
                elif random.random() < 0.1:
                    self.target_location = random.choice([self.school_coords, self.store_coords, self.work_coords])
                else:
                    if self.age <= 35:
                        self.target_location = self.school_coords
                    elif self.age > 80:
                        self.target_location = self.store_coords
                    else:
                        self.target_location = self.work_coords if random.random() < 0.7 else self.store_coords
            else:
                self.target_location = self.home_coords
        else:
            self.target_location = self.home_coords


    def __repr__(self):
        return 'Person: ' + self.condition + '  @ ' + str(self.x) + ', ' + str(self.y)
